@@ -107,6 +107,31 @@ class UpdatePolicyTarget(neutronV20.UpdateCommand):
     resource = 'policy_target'
     log = logging.getLogger(__name__ + '.UpdatePolicyTarget')
 
+    def add_known_arguments(self, parser):
+        parser.add_argument(
+            '--description',
+            help=_('Description of the Policy Target'))
+        parser.add_argument(
+            '--policy-target-group', metavar='PTG',
+            default='',
+            help=_('Policy Target Group uuid'))
+        parser.add_argument(
+            '--name',
+            help=_('New name of the Policy Target'))
+
+    def args2body(self, parsed_args):
+        body = {self.resource: {}, }
+
+        neutronV20.update_dict(parsed_args, body[self.resource],
+                               ['name', 'tenant_id', 'description'])
+        if parsed_args.policy_target_group:
+            body[self.resource]['policy_target_group_id'] = \
+                neutronV20.find_resourceid_by_name_or_id(
+                    self.get_client(), 'policy_target_group',
+                    parsed_args.policy_target_group)
+
+        return body
+
 
 class ListPolicyTargetGroup(neutronV20.ListCommand):
     """List Policy Target Groups that belong to a given tenant."""
@@ -157,6 +182,9 @@ class CreatePolicyTargetGroup(neutronV20.CreateCommand):
         parser.add_argument(
             '--subnets', type=string.split,
             help=_('List of neutron subnet uuids'))
+        parser.add_argument(
+            '--shared', type=bool,
+            help=_('Shared flag'))
 
     def args2body(self, parsed_args):
         body = {self.resource: {}, }
@@ -189,17 +217,10 @@ class CreatePolicyTargetGroup(neutronV20.CreateCommand):
                 parsed_args.consumed_policy_rule_sets[id_key] = \
                     parsed_args.consumed_policy_rule_sets.pop(key)
 
-        if parsed_args.subnets:
-            for subnet in parsed_args.subnets:
-                subnet_id = neutronV20.find_resourceid_by_name_or_id(
-                    self.get_client(), 'subnet',
-                    subnet)
-                parsed_args.subnets.remove(subnet)
-                parsed_args.subnets.append(subnet_id)
         neutronV20.update_dict(parsed_args, body[self.resource],
                                ['name', 'tenant_id', 'description',
                                 'provided_policy_rule_sets', 'subnets',
-                                'consumed_policy_rule_sets'])
+                                'consumed_policy_rule_sets', 'shared'])
 
         return body
 
@@ -235,7 +256,10 @@ class UpdatePolicyTargetGroup(neutronV20.UpdateCommand):
             help=_('Dictionary of consumed policy rule set uuids'))
         parser.add_argument(
             '--subnets', type=string.split,
-            help=_('Subnet for the Policy Target Group'))
+            help=_('List of neutron subnet uuids'))
+        parser.add_argument(
+            '--shared', type=bool,
+            help=_('Shared flag'))
 
     def args2body(self, parsed_args):
         body = {self.resource: {}, }
@@ -268,18 +292,10 @@ class UpdatePolicyTargetGroup(neutronV20.UpdateCommand):
                 parsed_args.consumed_policy_rule_sets[id_key] = \
                     parsed_args.consumed_policy_rule_sets.pop(key)
 
-        if parsed_args.subnets:
-            for subnet in parsed_args.subnets:
-                subnet_id = neutronV20.find_resourceid_by_name_or_id(
-                    self.get_client(), 'subnet',
-                    subnet)
-                parsed_args.subnets.remove(subnet)
-                parsed_args.subnets.append(subnet_id)
-
         neutronV20.update_dict(parsed_args, body[self.resource],
                                ['name', 'tenant_id', 'description',
                                 'provided_policy_rule_sets', 'subnets',
-                                'consumed_policy_rule_sets'])
+                                'consumed_policy_rule_sets', 'shared'])
 
         return body
 
@@ -314,7 +330,7 @@ class CreateL2Policy(neutronV20.CreateCommand):
             help=_('Description of the L2 Policy'))
         parser.add_argument(
             '--network',
-            help=_('Network to map the L2 Policy'))
+            help=_('Neutron network uuid to map the L2 Policy to'))
         parser.add_argument(
             '--l3-policy',
             default='',
@@ -322,22 +338,24 @@ class CreateL2Policy(neutronV20.CreateCommand):
         parser.add_argument(
             'name', metavar='NAME',
             help=_('Name of L2 Policy to create'))
+        parser.add_argument(
+            '--shared', type=bool,
+            help=_('Shared flag'))
 
     def args2body(self, parsed_args):
         body = {self.resource: {}, }
 
         neutronV20.update_dict(parsed_args, body[self.resource],
-                               ['name', 'tenant_id', 'description'])
+                               ['name', 'tenant_id', 'description', 'shared'])
         if parsed_args.l3_policy:
             body[self.resource]['l3_policy_id'] = \
                 neutronV20.find_resourceid_by_name_or_id(
                     self.get_client(), 'l3_policy',
                     parsed_args.l3_policy)
         if parsed_args.network:
-            body[self.resource]['network_id'] = \
-                neutronV20.find_resourceid_by_name_or_id(
-                    self.get_client(), 'network',
-                    parsed_args.network)
+            body[self.resource]['network_id'] = (
+                parsed_args.network)
+
         return body
 
 
@@ -354,6 +372,34 @@ class UpdateL2Policy(neutronV20.UpdateCommand):
     resource = 'l2_policy'
     log = logging.getLogger(__name__ + '.UpdateL2Policy')
 
+    def add_known_arguments(self, parser):
+        parser.add_argument(
+            '--description',
+            help=_('Description of the L2 Policy'))
+        parser.add_argument(
+            '--l3-policy',
+            default='',
+            help=_('L3 Policy uuid'))
+        parser.add_argument(
+            '--name', metavar='NAME',
+            help=_('New name of the L2 Policy'))
+        parser.add_argument(
+            '--shared', type=bool,
+            help=_('Shared flag'))
+
+    def args2body(self, parsed_args):
+        body = {self.resource: {}, }
+
+        neutronV20.update_dict(parsed_args, body[self.resource],
+                               ['name', 'tenant_id', 'description', 'shared'])
+        if parsed_args.l3_policy:
+            body[self.resource]['l3_policy_id'] = \
+                neutronV20.find_resourceid_by_name_or_id(
+                    self.get_client(), 'l3_policy',
+                    parsed_args.l3_policy)
+
+        return body
+
 
 class ListL3Policy(neutronV20.ListCommand):
     """List l3_policies that belong to a given tenant."""
@@ -405,6 +451,9 @@ class CreateL3Policy(neutronV20.CreateCommand):
         parser.add_argument(
             'name', metavar='NAME',
             help=_('Name of L3 policy to create'))
+        parser.add_argument(
+            '--shared', type=bool,
+            help=_('Shared flag'))
 
     def args2body(self, parsed_args):
         body = {self.resource: {}, }
@@ -423,7 +472,7 @@ class CreateL3Policy(neutronV20.CreateCommand):
         neutronV20.update_dict(parsed_args, body[self.resource],
                                ['name', 'tenant_id', 'description',
                                 'ip_version', 'ip_pool',
-                                'subnet_prefix_length'])
+                                'subnet_prefix_length', 'shared'])
 
         return body
 
@@ -463,7 +512,10 @@ class UpdateL3Policy(neutronV20.UpdateCommand):
                    '(this option can be repeated)'))
         parser.add_argument(
             '--name', metavar='NAME',
-            help=_('Name of L3 policy to create'))
+            help=_('New name of the L3 Policy'))
+        parser.add_argument(
+            '--shared', type=bool,
+            help=_('Shared flag'))
 
     def args2body(self, parsed_args):
         body = {self.resource: {}, }
@@ -471,6 +523,8 @@ class UpdateL3Policy(neutronV20.UpdateCommand):
         if parsed_args.external_segments:
             external_segments_dict = {}
             for external_segment in parsed_args.external_segments:
+                if not external_segment:
+                    break
                 external_segment_id = neutronV20.find_resourceid_by_name_or_id(
                     self.get_client(), 'external_segment',
                     external_segment.keys()[0])
@@ -482,7 +536,7 @@ class UpdateL3Policy(neutronV20.UpdateCommand):
         neutronV20.update_dict(parsed_args, body[self.resource],
                                ['name', 'tenant_id', 'description',
                                 'ip_version', 'ip_pool',
-                                'subnet_prefix_length'])
+                                'subnet_prefix_length', 'shared'])
 
         return body
 
@@ -525,25 +579,16 @@ class CreateNetworkServicePolicy(neutronV20.CreateCommand):
             type=utils.str2dict,
             help=_('Network service params for this network service policy'
                    '(This option can be repeated).'))
+        parser.add_argument(
+            '--shared', type=bool,
+            help=_('Shared flag'))
 
     def args2body(self, parsed_args):
         body = {self.resource: {}, }
 
-        """
-        if parsed_args.name:
-            body[self.resource].update({'name': parsed_args.name})
-
-        if parsed_args.description:
-            body[self.resource].update({'description': parsed_args.name})
-
-        if parsed_args.network_service_params:
-            body[self.resource]['network_service_params'] = (
-                parsed_args.network_sercice_params)
-        """
-
         neutronV20.update_dict(parsed_args, body[self.resource],
                                ['name', 'tenant_id', 'description',
-                                'network_service_params'])
+                                'network_service_params', 'shared'])
         return body
 
 
@@ -574,13 +619,16 @@ class UpdateNetworkServicePolicy(neutronV20.UpdateCommand):
             type=utils.str2dict,
             help=_('Network service params for this network service policy'
                    '(This option can be repeated).'))
+        parser.add_argument(
+            '--shared', type=bool,
+            help=_('Shared flag'))
 
     def args2body(self, parsed_args):
         body = {self.resource: {}, }
 
         neutronV20.update_dict(parsed_args, body[self.resource],
                                ['name', 'tenant_id', 'description',
-                                'network_service_params'])
+                                'network_service_params', 'shared'])
         return body
 
 
@@ -626,13 +674,17 @@ class CreatePolicyClassifier(neutronV20.CreateCommand):
         parser.add_argument(
             'name', metavar='NAME',
             help=_('Name of classifier to create'))
+        parser.add_argument(
+            '--shared', type=bool,
+            help=_('Shared flag'))
 
     def args2body(self, parsed_args):
         body = {self.resource: {}, }
 
         neutronV20.update_dict(parsed_args, body[self.resource],
                                ['name', 'tenant_id', 'description',
-                                'protocol', 'port_range', 'direction'])
+                                'protocol', 'port_range',
+                                'direction', 'shared'])
 
         return body
 
@@ -650,6 +702,38 @@ class UpdatePolicyClassifier(neutronV20.UpdateCommand):
     resource = 'policy_classifier'
     log = logging.getLogger(__name__ + '.UpdatePolicyClassifier')
 
+    def add_known_arguments(self, parser):
+        parser.add_argument(
+            '--description',
+            help=_('Description of the policy classifier'))
+        parser.add_argument(
+            '--protocol',
+            choices=['tcp', 'udp', 'icmp'],
+            help=_('Protocol'))
+        parser.add_argument(
+            '--port-range',
+            help=_('Port range'))
+        parser.add_argument(
+            '--direction',
+            choices=['in', 'out', 'bi', ''],
+            help=_('Direction'))
+        parser.add_argument(
+            '--name',
+            help=_('New name of the classifier'))
+        parser.add_argument(
+            '--shared', type=bool,
+            help=_('Shared flag'))
+
+    def args2body(self, parsed_args):
+        body = {self.resource: {}, }
+
+        neutronV20.update_dict(parsed_args, body[self.resource],
+                               ['name', 'tenant_id', 'description',
+                                'protocol', 'port_range',
+                                'direction', 'shared'])
+
+        return body
+
 
 class ListPolicyAction(neutronV20.ListCommand):
     """List actions that belong to a given tenant."""
@@ -688,6 +772,9 @@ class CreatePolicyAction(neutronV20.CreateCommand):
         parser.add_argument(
             'name', metavar='NAME',
             help=_('Name of action to create'))
+        parser.add_argument(
+            '--shared', type=bool,
+            help=_('Shared flag'))
 
     def args2body(self, parsed_args):
         body = {self.resource: {}, }
@@ -700,7 +787,7 @@ class CreatePolicyAction(neutronV20.CreateCommand):
 
         neutronV20.update_dict(parsed_args, body[self.resource],
                                ['name', 'tenant_id', 'description',
-                                'action_type'])
+                                'action_type', 'shared'])
 
         return body
 
@@ -718,6 +805,35 @@ class UpdatePolicyAction(neutronV20.UpdateCommand):
     resource = 'policy_action'
     log = logging.getLogger(__name__ + '.UpdatePolicyAction')
 
+    def add_known_arguments(self, parser):
+        parser.add_argument(
+            '--description',
+            help=_('Description of the policy action'))
+        parser.add_argument(
+            '--action-value',
+            help=_('Name/UUID of servicechain spec for redirect action'))
+        parser.add_argument(
+            '--name',
+            help=_('New name of the action'))
+        parser.add_argument(
+            '--shared', type=bool,
+            help=_('Shared flag'))
+
+    def args2body(self, parsed_args):
+        body = {self.resource: {}, }
+
+        if parsed_args.action_value:
+            body[self.resource]['action_value'] = (
+                neutronV20.find_resourceid_by_name_or_id(
+                    self.get_client(), 'servicechain_spec',
+                    parsed_args.action_value))
+
+        neutronV20.update_dict(parsed_args, body[self.resource],
+                               ['name', 'tenant_id', 'description',
+                                'shared'])
+
+        return body
+
 
 class ListPolicyRule(neutronV20.ListCommand):
     """List policy_rules that belong to a given tenant."""
@@ -760,6 +876,9 @@ class CreatePolicyRule(neutronV20.CreateCommand):
         parser.add_argument(
             'name', metavar='NAME',
             help=_('Name of policy_rule to create'))
+        parser.add_argument(
+            '--shared', type=bool,
+            help=_('Shared flag'))
 
     def args2body(self, parsed_args):
         body = {self.resource: {}, }
@@ -780,7 +899,7 @@ class CreatePolicyRule(neutronV20.CreateCommand):
 
         neutronV20.update_dict(parsed_args, body[self.resource],
                                ['name', 'tenant_id', 'description',
-                                'enabled'])
+                                'enabled', 'shared'])
 
         return body
 
@@ -808,6 +927,9 @@ class UpdatePolicyRule(neutronV20.UpdateCommand):
         parser.add_argument(
             '--actions', type=string.split,
             help=_('List of policy actions'))
+        parser.add_argument(
+            '--shared', type=bool,
+            help=_('Shared flag'))
 
     def args2body(self, parsed_args):
         body = {self.resource: {}, }
@@ -828,7 +950,7 @@ class UpdatePolicyRule(neutronV20.UpdateCommand):
 
         neutronV20.update_dict(parsed_args, body[self.resource],
                                ['name', 'description',
-                                'enabled'])
+                                'enabled', 'shared'])
         return body
 
 
@@ -869,6 +991,9 @@ class CreatePolicyRuleSet(neutronV20.CreateCommand):
         parser.add_argument(
             'name', metavar='NAME',
             help=_('Name of policy rule set to create'))
+        parser.add_argument(
+            '--shared', type=bool,
+            help=_('Shared flag'))
 
     def args2body(self, parsed_args):
         body = {self.resource: {}, }
@@ -888,7 +1013,7 @@ class CreatePolicyRuleSet(neutronV20.CreateCommand):
                     elem) for elem in parsed_args.child_policy_rule_sets]
 
         neutronV20.update_dict(parsed_args, body[self.resource],
-                               ['name', 'tenant_id', 'description'])
+                               ['name', 'tenant_id', 'description', 'shared'])
         return body
 
 
@@ -912,6 +1037,9 @@ class UpdatePolicyRuleSet(neutronV20.UpdateCommand):
         parser.add_argument(
             '--child-policy-rule-sets', type=string.split,
             help=_('List of child policy rule sets'))
+        parser.add_argument(
+            '--shared', type=bool,
+            help=_('Shared flag'))
 
     def args2body(self, parsed_args):
         body = {self.resource: {}, }
@@ -933,7 +1061,7 @@ class UpdatePolicyRuleSet(neutronV20.UpdateCommand):
                 parsed_args.child_policy_rule_sets)
         neutronV20.update_dict(parsed_args, body[self.resource],
                                ['name', 'description', 'policy_rules',
-                                'child_policy_rule_sets'])
+                                'child_policy_rule_sets', 'shared'])
         return body
 
 
